(function (window, undefined) {
    var AM,
        ua = navigator.userAgent.toLowerCase(),
        isAndroid = /android/.test(ua),
        isFF = /firefox/.test(ua);

    window.AudioManager = AM = {
        volume: 50,
        muted: false,
        audio: {},
        paths: {
            mp3: "",
            ogg: ""
        },
        playing: {},
        i: 0,
        add: function (id, name) {
            this.audio[id] = load(name);
            this.i++;
        },
        play: function (list) {
            var me = this,
                can = function (e) {
                    var audio = this;
                    audio.removeEventListener('canplaythrough', can);
                    audio.initialized = true;
                    audio.currentTime = 0;
                    audio.muted = AM.muted;
                    audio.volume = AM.muted ? 0 : me.volume / 100;
                    audio.play();
                },
                audio,
                id;
            if (!(list instanceof Array)) {
                list = [list];
            }
            for (var i = 0, len = list.length; i < len; i++) {
                id = list[i];
                audio = me.audio[id];
                audio.initialized = false;
                me.playing[id] = audio;
                if (isAndroid || isFF) {
                    audio.initialized = true;
                    audio.currentTime = 0;
                    audio.muted = AM.muted;
                    audio.volume = AM.muted ? 0 : me.volume / 100;
                    audio.play();
                } else {
                    audio.addEventListener('canplaythrough', can.bind(audio), false);
                    audio.load();
                }
            }

        },
        stop: function (list) {
            var audio;
            if (list) {
                if (!(list instanceof Array)) {
                    list = [list];
                }
                for (var i = 0, len = list.length; i < len; i++) {
                    id = list[i];
                    audio = AM.playing[id];
                    audio.pause();
                    delete AM.playing[id];
                }
            } else {
                for (var id in AM.playing) {
                    if (AM.playing.hasOwnProperty(id)) {
                        audio = AM.playing[id];
                        if (audio.initialized) {
                            audio.pause();
                            delete AM.playing[id];

                        }
                    }
                }
            }
        },
        increase: function () {
            if (this.volume < 100) {
                this.volume += 10;
            }
            adjustVolume();
        },
        decrease: function () {
            if (this.volume > 0) {
                this.volume -= 10;
            }
        },
        mute: function () {
            this.muted = !this.muted;
            adjustVolume();
        },
        setPaths: function (obj) {
            if (obj.ogg) {
                this.paths.ogg = obj.ogg;
            }
            if (obj.mp3) {
                this.paths.mp3 = obj.mp3;
            }
        }
    };

    function load(name) {
        var audio = document.createElement('audio'),
            source = document.createElement('source');
        audio.setAttribute('preload', 'false');
        if (audio.canPlayType('audio/mpeg;')) {
            source.type = 'audio/mpeg';
            source.src = AM.paths.mp3 + '/' + name + '.mp3';
        } else {
            source.type = 'audio/ogg';
            source.src = AM.paths.ogg + '/' + name + '.ogg';
        }
        audio.appendChild(source);
        return audio;
    }

    function adjustVolume() {
        for (var id in AM.playing) {
            if (AM.playing.hasOwnProperty(id)) {
                AM.playing[id].volume = AM.volume / 100;
                AM.playing[id].muted = AM.muted;
            }
        }
    }
}(window));