var readyStateCheckInterval = setInterval(function () {
    if (document.readyState === "complete") {
        clearInterval(readyStateCheckInterval);
        main();
    }
}, 10);

/**
 * main function loop
 */
function main() {
    /**
     * init polyfills
     */
    CSS3.init();
    /**
     *
     * @type {*}
     */
    var layout = new View({
        config: {
            tpl: 'container',
            renderTo: 'body',
            models: window.models
        },
        init: function (opts) {
            View.prototype.init.apply(this, arguments);
            this.processData();
        },
        /**
         * process data from json
         */
        processData: function () {
            var models = this.get('config').models,
                questions = models["questions"],
                mainBackgroundColor = questions.background.color,
                mainBackgroundImage = questions.background.image,
                items = questions.items[0].items,
                me = this;
            me.set('questions', items);
            me.set('mainBackgroundColor', mainBackgroundColor);
            me.set('mainBackgroundImage', mainBackgroundImage);
        },
        fullyfill: function () {

            var container = document.getElementById('slide-container'),
                width = document.querySelector('body').offsetWidth,
                height = document.querySelector('body').offsetHeight,
                padding = (width - 1720) / 2,
                style = document.getElementById('question-style') || document.createElement('style');

            if (width <= 1920){
                style.innerHTML = "";
                return;
            }

            if(height < 1080){
                height = 1080;
            }

            style.setAttribute('id', 'question-style');
            style.setAttribute('type', 'text/css');
            style.innerHTML = ".question {" +
                "padding: 0 " + padding + "px;" +
                "height: " + height + "px;"
            "}";

            document.querySelector('body').appendChild(style);

            //todo: test

        },
        /**
         * generate all questions from data
         */
        generateQuestions: function () {
            var me = this,
                config = me.get('config'),
                models = config.models.questions.items,
                parent = me.get('el'),
                views = me.get('config').views,
                questions = 0,
                elements = 0,
                good,
                bad;

            good = new OverlayView({
                config: {
                    id: 'good',
                    tpl: 'good-answer',
                    renderTo: '#body'
                }
            });
            good.render();

            bad = new OverlayView({
                config: {
                    id: 'bad',
                    tpl: 'bad-answer',
                    renderTo: '#body'
                }
            });
            bad.render();

            for (var i = 0, len = models.length; i < len; i++) {
                switch (models[i].type) {
                    case 'question' :
                    {
                        var question = new QuestionView({
                            config: {
                                tpl: 'question',
                                parent: parent,
                                renderTo: '#question-container',
                                models: models[i]
                            }
                        });
                        question.render();
                        question.generateContent();
                        views[question.get('config').id] = question;
                        questions++;
                        elements++;
                        break;
                    }
                    case 'intro' :
                    {
                        var overlay = new OverlayView({
                            config: {
                                id: 'intro',
                                tpl: 'intro',
                                renderTo: '#body',
                                models: models[i]
                            }
                        });
                        overlay.render();
                        views[overlay.get('config').id] = overlay;
                        elements++;
                        break;
                    }
                    case 'outro' :
                    {
                        var outro = new OverlayView({
                            config: {
                                id: 'outro',
                                tpl: 'outro',
                                renderTo: '#body',
                                models: models[i]
                            }
                        });
                        outro.render();
                        views[outro.get('config').id] = outro;
                        elements++;
                        break;
                    }
                }
            }
            this.set('numbOfElements', elements);
            this.set('numbOfQuestions', questions);
            me.fullyfill();
            parent.querySelector('#question-container').style.width = (questions * 1920) + 'px';
        }
    });
    /**
     * render template
     */
    layout.render();
    /**
     * and generate all questions
     */
    layout.generateQuestions();
    /**
     *
     * @type {*}
     */
    var ctl = new Controller({
        config: {
            name: 'Main',
            views: {
                layout: ViewManager.get('view-0')
            },
            current: 0,
            isAnimated: false,
            completed: [],
            volume: 50,
            mute: false
        },
        /**
         *
         */
        init: function () {
            Controller.prototype.init.apply(this, arguments);
            this.initMobile();
            this.bindElements();
            this.bindEvents();
            this.bindDelegates();
            this.initAudio();
            this.initImages();
            this.getRouter().restore();
            this.startPage();
            this.fullyfill();
        },
        initMobile: function () {
            this.portraitMobileMode();
        },
        initAudio: function () {
            var me = this;
            me.set('toLoad', 1);
            AudioManager.setPaths({
                ogg: 'assets/audio/ogg',
                mp3: 'assets/audio/mp3'
            });
            AudioManager.add('voGood1', 'VO/bardzo_dobrze');
            AudioManager.add('voBad1', 'VO/niestety');
            AudioManager.add('soGood', 'SD/succes');
            AudioManager.add('soBad', 'SD/failure');
            AudioManager.add('start', 'SD/game_start');
            AudioManager.add('click', 'SD/ball_click');
            setTimeout(function(){
                me.fireEvent('soundsLoaded');
            }, 200);
//            $.mbAudio.sounds = {
//                start: {
//                    id: 'start',
//                    ogg: "assets/audio/ogg/SD/game_start.ogg",
//                    mp3: "assets/audio/mp3/SD/game_start.mp3",
//                    //example of sprite
//                    sprite: {
//                        start: {id: "start", start: 0, end: 1.776327, loop: false}
//                    }
//                },
//                click: {
//                    id: 'click',
//                    ogg: "assets/audio/ogg/SD/ball_click.ogg",
//                    mp3: "assets/audio/mp3/SD/ball_click.mp3",
//                    //example of sprite
//                    sprite: {
//                        click: {id: "click", start: 0, end: 0.287347, loop: false}
//                    }
//                },
//                soBad: {
//                    id: 'soBad',
//                    ogg: "assets/audio/ogg/SD/failure.ogg",
//                    mp3: "assets/audio/mp3/SD/failure.mp3",
//                    //example of sprite
//                    sprite: {
//                        soBad: {id: "soBad", start: 0, end: 1, loop: false}
//                    }
//                },
//                soGood: {
//                    id: 'soGood',
//                    ogg: "assets/audio/ogg/SD/succes.ogg",
//                    mp3: "assets/audio/mp3/SD/succes.mp3",
//                    //example of sprite
//                    sprite: {
//                        soGood: {id: "soGood", start: 0, end: 1, loop: false}
//                    }
//                },
//                voBad1: {
//                    id: 'voBad1',
//                    ogg: "assets/audio/ogg/VO/niestety.ogg",
//                    mp3: "assets/audio/mp3/VO/niestety.mp3",
//                    //example of sprite
//                    sprite: {
//                        voBad1: {id: "voBad1", start: 0, end: 2, loop: false}
//                    }
//                },
//                voGood1: {
//                    id: 'voGood1',
//                    ogg: "assets/audio/ogg/VO/bardzo_dobrze.ogg",
//                    mp3: "assets/audio/mp3/VO/bardzo_dobrze.mp3",
//                    //example of sprite
//                    sprite: {
//                        voGood1: {id: "voGood1", start: 0, end: 1, loop: false}
//                    }
//                }
//            };
//            $(document).on("soundsLoaded", function () {
//                me.fireEvent('soundsLoaded');
//            });
//            $.mbAudio.preload();
        },
        initImages: function () {
            var me = this,
                images = document.querySelectorAll('img'),
                counter = 0,
                loaded = function () {
                    counter--;
                    if (counter === 0) {
                        me.fireEvent('imagesLoaded');
                    }
                };
            me.set('toLoad', 2);
            for (var i = 0, l = images.length; i < l; i++) {
                if (!images[i].complete) {
                    counter++;
                    $(images[i]).on('load', loaded)
                }
            }
            if (counter === 0)
                me.fireEvent('imagesLoaded');
        },
        startPage: function () {
            var intro = ViewManager.get('intro'),
                good = ViewManager.get('good'),
                bad = ViewManager.get('bad');
            this.changeQuestion(0);
            jQuery(good.get('el')).css('display', 'none');
            jQuery(bad.get('el')).css('display', 'none');
        },
        /**
         * bind all events
         */
        bindEvents: function () {
            var me = this,
                start = me.get('$startBtn'),
                mute = me.get('$mute'),
                bad = ViewManager.get('bad').get('el'),
                btnRight = me.get('$btnRight'),
                volumeUp = me.get('$volumeUp'),
                volumeDown = me.get('$volumeDown');
            start.addEventListener('click', this.onStartClick.bind(me), false);
            mute.addEventListener('click', this.onMuteClick.bind(me), false);
            bad.addEventListener('click', this.onBadClick.bind(me), false);
            btnRight.addEventListener('click', this.onRightArrowClick.bind(me), false);
            volumeUp.addEventListener('click', this.onVolumeUpClick.bind(me), false);
            volumeDown.addEventListener('click', this.onVolumeDownClick.bind(me), false);
            window.addEventListener('keydown', this.onKeyDown.bind(me), false);
            window.addEventListener('resize', this.fullyfill.bind(me), false);
            window.addEventListener('resize', layout.fullyfill.bind(layout));
            me.addListener('imagesLoaded', me.engineReady.bind(me));
            me.addListener('soundsLoaded', me.engineReady.bind(me));
        },
        bindDelegates: function () {
            var body = jQuery('body');
            body.delegate('#refresh', 'click', this.onRefreshBtnClick.bind(this));
            body.delegate('li.answer', 'click', this.onAnswerClick.bind(this));
        },
        bindElements: function () {
            var me = this;
            me.set('$startBtn', document.getElementById('start'));
            me.set('$mainUpper', jQuery('#question-upper-container'));
            me.set('$mute', document.getElementById('mute'));
            me.set('$volumeUp', document.getElementById('volume-plus'));
            me.set('$volumeDown', document.getElementById('volume-minus'));
            me.set('$btnRight', document.getElementById('arrow-right'));
            me.set('$loading', document.getElementById('loading'));
        },
        engineReady: function () {
            var me = this,
                toLoad = me.get('toLoad'),
                $l,
                $s;
            toLoad--;
            me.set('toLoad', toLoad);
            if (toLoad === 0) {
                $l = $(me.get('$loading'));
                $s = $(me.get('$startBtn'));
                $l.animate({opacity: 0}, function () {
                    $l.css('display', 'none');
                    $s.css({opacity: 0, display: 'inline'}).animate({opacity: 1});
                });
            }

        },
        onVolumeUpClick: function () {
            var config = this.get('config');
            AudioManager.play('click');
//            $.mbAudio.stopAllSounds();
//            $.mbAudio.play('click');
            if (config.mute) {
                this.onMuteClick();
            }
            if (config.volume < 100) {
                config.volume += 10;
                AudioManager.increase();
            }
        },
        onVolumeDownClick: function () {
            var config = this.get('config');

            AudioManager.play('click');
//            $.mbAudio.stopAllSounds();
//            $.mbAudio.play('click');
            if (config.volume > 0) {
                config.volume -= 10;
                AudioManager.decrease();
//                $.mbAudio.setVolumeAllSounds(config.volume);
            }
            if (config.volume === 0) {
                if (!config.mute) {
                    this.onMuteClick();
                }
            }
        },
        /**
         * mute all sounds
         * change icon
         */
        onMuteClick: function () {
            var mute = this.get('$mute'),
                config = this.get('config');

            AudioManager.play('click');
//            $.mbAudio.stopAllSounds();
//            $.mbAudio.play('click');
            if (config.mute) {
                config.mute = false;
                mute.className = mute.className.replace(" pressed", "");
//                $.mbAudio.unMuteAllSounds();
                AudioManager.mute();
            } else {
                mute.className += " pressed";
                config.mute = true;
//                $.mbAudio.muteAllSounds();
                AudioManager.mute();
            }
        },
        onBadClick: function (e) {
            var me = this,
                bad = ViewManager.get('bad'),
                badTimeout = me.get('badHideTimeout'),
                badAnswer = document.querySelector('.answer-invalid'),
                badAnswerView;
            if (!badAnswer) {
                return;
            }
            badAnswerView = ViewManager.get(badAnswer.getAttribute('yamvc-id'));
            if (badTimeout) {
                clearTimeout(badTimeout);
            }
            badAnswerView.clear();
            bad.hide(function () {
            });
        },
        /**
         * when one of the answers was chosen
         * @param e
         */
        onAnswerClick: function (e) {
            var me = this,
                id = e.target.getAttribute('yamvc-id'),
                view = ViewManager.get(id),
                config = me.get('config'),
                parent = view.get('config').parent,
                childView,
                btnRight,
                good,
                bad,
                len;

            if (!parent.correct) {
                for (var key in parent.views) {
                    if (parent.views.hasOwnProperty(key)) {
                        childView = parent.views[key];
                        if (childView.get('config').models.correct) {
                            parent.correct = parent.correct || 0;
                            parent.correct++;
                        }
                    }
                }
            }
            // prevent for multiple clicks
            if (config.isAnimated) {
                return;
            }
            // if chosen answer is valid
            if (view.isValid()) {
                if (view.isSelected()) {
                    return;
                }
                parent.correct--;
                // play success sounds
                AudioManager.stop();
                AudioManager.play(['soGood', 'voGood1']);
//                $.mbAudio.stopAllSounds();
//                $.mbAudio.play('soGood');
//                $.mbAudio.play('voGood1');
                // mark is as valid
                view.markValid();
                // get overlay
                good = ViewManager.get('good');
                // set last chosen answer
                good.set('lastAnswer', view);
                good.set('currentView', parent);
                // get right button element
                if (!parent.correct) {
                    me.set('goNext', true);
                    btnRight = me.get('$btnRight');
                    // show it via animate
                    jQuery(btnRight).css({
                        display: 'block',
                        opacity: 0
                    }).animate({
                            opacity: 1
                        }, 500);
                    // use built in function to show overlay
                    // too
                    good.show();
                    // count answered questions and add
                    // another to array
                    len = config.completed.length + 1;
                    config.completed.push(len);
                }
            } else {
                // play audio
                AudioManager.stop();
                AudioManager.play(['soBad', 'voBad1']);
//                $.mbAudio.stopAllSounds();
//                $.mbAudio.play('soBad');
//                $.mbAudio.play('voBad1');
                // get overlay for bad answer
                bad = ViewManager.get('bad');
                // mark answer as invalid
                view.markInvalid();
                // show overlay
                bad.show(function () {
                    // hide it after timeout
                    me.set('badHideTimeout',
                        setTimeout(function () {
                            bad.hide();
                            view.clear();
                        }, 5000)
                    );
                });
            }
        },
        /**
         * on left arrow click
         * @param e
         */
        onLeftArrowClick: function (e) {
            if (e)
                e.preventDefault();
            if (this.get('config').isAnimated) {
                return true;
            }
            this.get('config').current--;
            if (parseInt(this.get('config').current) < 0) {
                this.get('config').current = 0;
            }
            //todo: replace redirect
            //this.redirectTo("#question/" + this.get('config').current);
        },
        /**
         * on right arrow click
         * @param e
         */
        onRightArrowClick: function (e) {
            var me = this,
                config = me.get('config'),
                good = ViewManager.get('good'),
                btnRight = me.get('$btnRight'),
                lastA = good.get('lastAnswer'),
                views = lastA ? lastA.get('config').parent.views : null;

            if (typeof me.get('goNext') === 'undefined' || me.get('goNext') === false) {
                return true;
            }

            if (e)
                e.preventDefault();


            AudioManager.play('click');
//            $.mbAudio.stopAllSounds();
//            $.mbAudio.play('click');
            if (config.isAnimated) {
                return true;
            }

            me.set('goNext', false);
            jQuery(btnRight).animate({
                opacity: 0
            }, 500).css({
                    display: 'none',
                    opacity: 0
                });
            good.hide(function () {
                for (var i in views) {
                    if (views.hasOwnProperty(i)) {
                        views[i].clear();
                    }
                }
            });
            this.changeQuestion(config.completed.length + 1);
        },
        onStartClick: function (e) {
            if (this.get('config').isAnimated) {
                return true;
            }
            var $el = jQuery(ViewManager.get('intro').get('el')),
                me = this;
            this.get('config').isAnimated = true;
            $el.animate({opacity: 0}, 500, function () {
                jQuery(this).css('display', 'none');
                me.get('config').isAnimated = false;
            });
            this.get('config').current++;
            this.changeQuestion(1);

            AudioManager.play('start');
//            $.mbAudio.play('start');
        },
        onRefreshBtnClick: function () {
            var me = this,
                config = me.get('config');

            AudioManager.play('click');
//            $.mbAudio.stopAllSounds();
//            $.mbAudio.play('click');
            $('#question-upper-container').css('opacity', 0);
            config.current = 0;
            config.completed = [];
            //todo: replace redirect
            this.changeQuestion(0);
            //me.redirectTo("#question/" + config.current);
        },
        onKeyDown: function (e) {
            //key right
            if (e.keyCode == 39) {
                this.onRightArrowClick();
            } else
            // key left
            if (e.keyCode == 37) {
                this.onLeftArrowClick();
            }
        },
        /**
         * change question
         * @param id
         */
        portraitMobileMode: function () {
            var media,
                realWidth,
                mobile;
            if (window.innerHeight > window.innerWidth) {
                media = window.matchMedia('(max-device-width: 767px)').matches;
                mobile = isMobile();
                realWidth = (jQuery(window).width() / (window.devicePixelRatio || 1)) < 768;
                if (media || mobile && realWidth) {
                    this.fullyfillMobile();
                    this.set('portraitMobileMode', true);
                } else {
                    this.set('portraitMobileMode', false);
                    this.resetFullyfillMobile();
                }
            } else {
                this.set('portraitMobileMode', false);
                this.resetFullyfillMobile();
            }
        },
        changeQuestion: function (id) {
            var me = this,
                config = me.get('config'),
                layout = config.views.layout,
                widthT = document.querySelector('body').offsetWidth,
                width = widthT > 1920 ? widthT : 1920,
                container = layout.get('el').querySelector('#question-container'),
                max = config.views.layout.get('numbOfElements') - 1,
                good = ViewManager.get('good'),
                el;
            id = parseInt(id);
            if (id !== 0 && (config.completed.length + 1) !== id) {
                //todo: replace redirect
                //me.redirectTo("#question/" + (config.completed.length + 1));
                return;
            }
            config.current = id;
            // show container if not first page
            if (id > 0) {
                me.get('$mainUpper').css('opacity', 1);
            }
            config.isAnimated = true;
            // animate question
            if ((max - 1) === id) {
                el = good.get('el').querySelector('.addnotation');
                if (el) {
                    el.style.display = "none";
                }
            } else if (id === 0) {
                el = good.get('el').querySelector('.addnotation');
                if (el) {
                    el.style.display = "block";
                }
            }
            jQuery(container).animate({
                opacity: 0
            }, 500, function () {
                jQuery(this)
                    .css('transform', "translate(" + (-width * (id - 1)) + "px, 0px)")
                    .animate({
                        opacity: 1
                    }, 500, function () {
                        config.isAnimated = false;
                    });

                // if first page
                if (config.current === 0) {
                    //show intro
                    var intro = ViewManager.get('intro');
                    if (intro.isInDOM()) {
                        jQuery(intro.get('el')).css('display', 'block');
                        jQuery(intro.get('el')).animate({opacity: 1}, 1000, function () {
                        });
                    }
                }
            });
            var outro = ViewManager.get('outro');
            // if last page
            if (config.current === max) {
                // show outro
                jQuery(outro.get('el')).css('display', 'block');
                jQuery(outro.get('el')).animate({
                    opacity: 1
                }, 500, function () {

                });
            } else {
                // else hide it
                if (jQuery(outro.get('el')).css('display') === 'block') {
                    jQuery(outro.get('el')).animate({
                        opacity: 0
                    }, 500, function () {
                        jQuery(outro.get('el')).css('display', 'none');
                    });
                }
                me.fullyfill();
            }
        },
        /**
         * match content to height and width
         */

        fullyfill: function () {
            var len = this.get('config').views.layout.get('questionsCount') - 1;
            // if last page do nothing
            if (this.get('config').current === len) {
                return;
            }
            this.portraitMobileMode();
            // if mobile portrait use other fullyfill
            if (this.get('portraitMobileMode')) {
                return;
            }
            var containerUpper = layout.get('el').querySelectorAll('#question-upper-container').item(0),
                questions = layout.get('el').querySelectorAll('.question'),
                ratioX = 1,
                ratioY = 1,
                ratio,
                diffX = 0,
                diffY = 0,
                offsetHeight = containerUpper.offsetHeight - (isMobile() ? (30) : 0),
                i = this.get('config').current - 1;
            if (i < 0) {
                return;
            }
            var question = questions.item(i);
            if (!question) {
                return;
            }
            if (offsetHeight < question.offsetHeight) {
                ratioY = offsetHeight / question.offsetHeight;
                diffY = offsetHeight - question.offsetHeight;
            }
            if (containerUpper.offsetWidth < question.offsetWidth) {
                diffX = containerUpper.offsetWidth - question.offsetWidth;
                ratioX = containerUpper.offsetWidth / question.offsetWidth;
            }
            ratio = ratioY < ratioX ? ratioY : ratioX;
            if (ratio < 1) {
                if (diffX < 0 || diffY < 0) {
                    question.style[CSS3.get('transform')] = "translate" + (CSS3.has3d() ? "3d(" + ((diffX / 2) ) + "px, " + ((diffY / 2)) + "px, 0)" : "(" + ((diffX / 2) ) + "px, " + ((diffY / 2)) + "px)") + " scale(" + ratio + ")";

                } else {
                    question.style[CSS3.get('transform')] = "scale(" + ratio + ")";
                }
            } else {
                question.style[[CSS3.get('transform')]] = "";
            }
        },
        fullyfillMobile: function () {
            // run only when needed
            if (this.get('fullyfillMobile')) {
                return;
            }
            var nodes = document.querySelectorAll('.question'),
                screenH = jQuery(window).height(),
                screenW = jQuery(window).width(),
                width = 900,
                paddingLeft = 60,
                paddingRight = 1020,
                css = {
                    paddingLeft: paddingLeft,
                    paddingRight: paddingRight,
                    width: width - paddingLeft,
                    height: 1080
                };
            css[CSS3.get('transform')] = "";
            for (var i = 0, len = nodes.length; i < len; i++) {
                var question = nodes.item(i),
                    ratioX = 1,
                    ratioY = 1,
                    ratio,
                    diffX = 0,
                    diffY = 0;
                if (!question) {
                    return;
                }
                ratioX = screenW / (css.width + css.paddingLeft + 60);
                css.height = screenH / ratioX;
                jQuery(question).css(css);
                diffX = -(1920 - (ratioX * 1920));
                diffY = -(css.height - (ratioX * css.height));
                ratio = ratioY < ratioX ? ratioY : ratioX;
                if (ratio < 1) {
                    if (diffX < 0 || diffY < 0) {
                        question.style[CSS3.get('transform')] = "translate" + (CSS3.has3d() ? "3d(" + ((diffX / 2) ) + "px, " + ((diffY / 2)) + "px, 0)" : "(" + ((diffX / 2) ) + "px, " + ((diffY / 2)) + "px)") + " scale(" + ratio + ")";

                    } else {
                        question.style[CSS3.get('transform')] = "scale(" + ratio + ")";
                    }
                } else {
                    question.style[[CSS3.get('transform')]] = "";
                }
            }
            this.set('fullyfillMobile', true);
        },
        resetFullyfillMobile: function () {
            var css;
            // run only when needed
            if (!this.get('fullyfillMobile')) {
                return;
            }
            css = {
                paddingLeft: 300,
                paddingRight: 300,
                width: 1320,
                height: 1080
            };
            var nodes = document.querySelectorAll('.question');
            for (var i = 0, len = nodes.length; i < len; i++) {
                //another android bug simply removeAttribute('style')
                // doesnt work
                jQuery(nodes.item(i)).css(css);
            }
            this.set('fullyfillMobile', false);
        }
    });
}